/** @file tcl/timerscheduler.cc implementation of nub::scheduler using
    the tcl event scheduler */
///////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004-2007 University of Southern California
// Rob Peters <rjpeters at usc dot edu>
//
// created: Thu Oct 14 10:03:46 2004
// commit: $Id: timerscheduler.cc 10065 2007-04-12 05:54:56Z rjpeters $
// $HeadURL: file:///home/rjpeters/svnrepo/code/trunk/groovx/src/tcl/timerscheduler.cc $
//
// --------------------------------------------------------------------
//
// This file is part of GroovX.
//   [http://ilab.usc.edu/rjpeters/groovx/]
//
// GroovX is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// GroovX is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GroovX; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
///////////////////////////////////////////////////////////////////////

#ifndef GROOVX_TCL_TIMERSCHEDULER_CC_UTC20050628162421_DEFINED
#define GROOVX_TCL_TIMERSCHEDULER_CC_UTC20050628162421_DEFINED

#include "tcl/timerscheduler.h"

#include "tcl/eventloop.h"
#include "tcl/interp.h"

#include "rutz/shared_ptr.h"

#include <tcl.h>

#include "rutz/trace.h"
#include "rutz/debug.h"
GVX_DBG_REGISTER

namespace tcl
{
  class timer_scheduler_token;
}

class tcl::timer_scheduler_token : public nub::timer_token
{
public:
  timer_scheduler_token(int msec,
                        void (*callback)(void*),
                        void* clientdata);
  virtual ~timer_scheduler_token() throw();

private:
  static void c_callback(void* token) throw();

  typedef void (Callback)(void*);

  Tcl_TimerToken       m_token;
  Callback*      const m_callback;
  void*          const m_clientdata;
};

tcl::timer_scheduler_token::
timer_scheduler_token(int msec,
                      void (*callback)(void*),
                      void* clientdata)
  :
  m_token(Tcl_CreateTimerHandler(msec,
                                 c_callback,
                                 static_cast<void*>(this))),
  m_callback(callback),
  m_clientdata(clientdata)
{
GVX_TRACE("tcl::timer_scheduler_token::timer_scheduler_token");
}

tcl::timer_scheduler_token::~timer_scheduler_token() throw()
{
GVX_TRACE("tcl::timer_scheduler_token::~timer_scheduler_token");
  Tcl_DeleteTimerHandler(m_token);
}

void tcl::timer_scheduler_token::c_callback(void* token) throw()
{
  timer_scheduler_token* tok = static_cast<timer_scheduler_token*>(token);

  GVX_ASSERT(tok != 0);

  try
    {
      // BE CAREFUL: calling the client callback here may result in
      // the TimerScheduleToken object being destroyed. So, if we need
      // access to any of the token's member variables, we need to
      // make a local copy of them before calling the callback.
      (*tok->m_callback)(tok->m_clientdata);
    }
  catch(...)
    {
      tcl::event_loop::interp().handle_live_exception("timer callback",
                                                SRC_POS);
      tcl::event_loop::interp().background_error();
    }
}

tcl::timer_scheduler::timer_scheduler()
{
GVX_TRACE("tcl::timer_scheduler::timer_scheduler");
}

tcl::timer_scheduler::~timer_scheduler() throw()
{
GVX_TRACE("tcl::timer_scheduler::~timer_scheduler");
}

rutz::shared_ptr<nub::timer_token>
tcl::timer_scheduler::schedule(int msec,
                              void (*callback)(void*),
                              void* clientdata)
{
GVX_TRACE("tcl::timer_scheduler::schedule");
  // If the requested delay is zero -- i.e., immediate -- then don't
  // bother creating a timer handler. Instead, generate a direct
  // invocation; this saves a trip into the event loop and back.
  if (msec == 0)
    {
      (*callback)(clientdata);

      // Return a null pointer, representing the fact that there is no
      // pending callback in this case.
      return rutz::shared_ptr<nub::timer_token>();
    }

  return rutz::shared_ptr<nub::timer_token>
    (new tcl::timer_scheduler_token(msec, callback, clientdata));
}

static const char __attribute__((used)) vcid_groovx_tcl_timerscheduler_cc_utc20050628162421[] = "$Id: timerscheduler.cc 10065 2007-04-12 05:54:56Z rjpeters $ $HeadURL: file:///home/rjpeters/svnrepo/code/trunk/groovx/src/tcl/timerscheduler.cc $";
#endif // !GROOVX_TCL_TIMERSCHEDULER_CC_UTC20050628162421_DEFINED
